/*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// (C) PicoPeta Simputers Pvt. Ltd. (2003)
// Author: Santhosh N

#include "image.h"

#include <X11/xpm.h>
#include <stdlib.h>
#include <string.h>
#include "display.h"

void image_initialize_pixmap(px_pixmap* px)
{
	px->pxmp = None;
		
	px->mask = None;
		
	px->w = 0;
	px->h = 0;
}

void image_create_pixmap(Window win, px_pixmap* px, unsigned int w, unsigned int h)
{
	Display* display = display_get_Xdisplay();
	px->pxmp = XCreatePixmap(display, win, w, h, DefaultDepth(display, DefaultScreen(display)));
		
	px->mask = None;
		
	px->w = w;
	px->h = h;
}

Bool image_load_xpm(Window win, px_pixmap* px, char* path)
{
	XpmAttributes attrib;
	Display* display = display_get_Xdisplay();

	attrib.valuemask = 0; 

	if (px == NULL)
		return False;

	if (px->pxmp != None)
		XFreePixmap(display, px->pxmp);
	if (px->mask != None)
		XFreePixmap(display, px->mask);
	
	if (XpmReadFileToPixmap(display, win, path, &(px->pxmp), &(px->mask), &attrib) != XpmSuccess )
	{
          	fprintf(stderr, "failed loading image '%s'\n", path);
		return False;
	}
	else
	{
		px->h = attrib.height;
		px->w = attrib.width;
	}
	return True;
}

Bool image_load_file(Window win, px_pixmap* px, char* path)
{
	struct Image *img=NULL;
	int status;
	Display* display = display_get_Xdisplay();
	
	status = pimg_load(path, display, &img, SHAPE_MASK);

	if (status >= 0)
	{
		if (px->w != img->width || px->h != img->height)
		{
			if (px->pxmp != None)
			{
				XFreePixmap(display, px->pxmp);
				px->pxmp = None;
			}
			
			if (px->mask != None)
			{
				XFreePixmap(display, px->mask);
				px->mask = None;
			}
			px->w = img->width;
			px->h = img->height;
		}

		if (img->img != NULL)
		{
			GC gc;
			
			if (px->pxmp == None)
				px->pxmp = XCreatePixmap(display, win, px->w, px->h, 
						DefaultDepth(display, DefaultScreen(display)));

			gc = XCreateGC(display, px->pxmp, 0, NULL);
			XSetGraphicsExposures(display, gc, False);
			
			XPutImage(display, px->pxmp, gc, img->img, 0, 0, 0, 0, px->w, px->h);
			XFreeGC(display, gc);
		}

		if (img->mimg != NULL)
		{
			GC gc;
			
			if (px->mask == None)
				px->mask = XCreatePixmap(display, win, px->w, px->h, 1);

			gc = XCreateGC(display, px->mask, 0, NULL);
			XSetGraphicsExposures(display, gc, False);
			
			XPutImage(display, px->mask, gc, img->mimg, 0, 0, 0, 0, px->w, px->h);
			XFreeGC(display, gc);
		}
		pimg_free(&img);
		return True;
	}
	
	return False;
}

Bool image_copy_pixmap(Window win, px_pixmap* dst, const px_pixmap* src)
{
	GC gc, mask_gc;
	Display* display = display_get_Xdisplay();

	if (dst != NULL)
	{

/*		if (dst->w != src->w || dst->h != src->h)
		{
			if (dst->pxmp != None)
			{
				XFreePixmap(display, dst->pxmp);
				dst->pxmp = None;
			}

			if (dst->mask != None)
			{
				XFreePixmap(display, dst->mask);
				dst->mask = None;
			}
		}
*/			
		if (src->pxmp != None)
		{
			if (dst->pxmp == None)
			{
				dst->w = src->w;
				dst->h = src->h;
				dst->pxmp = XCreatePixmap(display, win, dst->w, dst->h, 
						DefaultDepth(display, DefaultScreen(display)));
			}
			gc = XCreateGC(display, dst->pxmp, 0 , NULL);
			XSetGraphicsExposures(display, gc, False);

			XCopyArea(display, src->pxmp, dst->pxmp, gc, 0, 0, src->w, src->h, 0, 0);
			XFreeGC(display, gc);
		}

		if (src->mask != None)
		{
			if (dst->mask == None)
				dst->mask = XCreatePixmap(display, win, dst->w, dst->h, 1);
			mask_gc = XCreateGC(display, dst->mask, 0 , NULL);
			XSetGraphicsExposures(display, mask_gc, False);
			
			XCopyArea(display, src->mask, dst->mask, mask_gc, 0, 0, src->w, src->h, 0, 0);
			XFreeGC(display, mask_gc);
		}
	
//		dst->h = src->h;
//		dst->w = src->w;
#ifdef __debug__
		XSync(display, False);
		printf("%s() ********* Copied from %p to %p with width = %d, height = %d\n", __func__, src, dst, src->w, src->h);
#endif
		return True;
	}
	return False;
}

Bool image_copy_image(Window win, px_pixmap* px, const struct Image* img)
{
	Display* display = display_get_Xdisplay();

	if (px != NULL && img != NULL)
	{
		if (img->img != NULL)
		{
			GC gc;
			
			if (px->pxmp == None)
				px->pxmp = XCreatePixmap(display, win, px->w, px->h, 
						DefaultDepth(display, DefaultScreen(display)));

			gc = XCreateGC(display, px->pxmp, 0, NULL);
			XSetGraphicsExposures(display, gc, False);
			
			XPutImage(display, px->pxmp, gc, img->img, 0, 0, 0, 0, px->w, px->h);
			XFreeGC(display, gc);
		}

		if (img->mimg != NULL)
		{
			GC gc;
			
			if (px->mask == None)
				px->mask = XCreatePixmap(display, win, px->w, px->h, 1);

			gc = XCreateGC(display, px->mask, 0, NULL);
			XSetGraphicsExposures(display, gc, False);
			
			XPutImage(display, px->mask, gc, img->mimg, 0, 0, 0, 0, px->w, px->h);
			XFreeGC(display, gc);
		}
		return True;
	}

	return False;
}

Bool image_copy_part_of_pixmap(Window win, px_pixmap* dst, const px_pixmap* src,
		int srcx, int srcy, int destx, int desty, int width, int height)
{
	GC gc, mask_gc;
	Display* display = display_get_Xdisplay();

	if (dst != NULL)
	{

/*		if (dst->w != src->w || dst->h != src->h)
		{
			if (dst->pxmp != None)
			{
				XFreePixmap(display, dst->pxmp);
				dst->pxmp = None;
			}

			if (dst->mask != None)
			{
				XFreePixmap(display, dst->mask);
				dst->mask = None;
			}
		}
*/			
		if (src->pxmp != None)
		{
			if (dst->pxmp == None)
			{
				dst->w = src->w;
				dst->h = src->h;
				dst->pxmp = XCreatePixmap(display, win, dst->w, dst->h, 
						DefaultDepth(display, DefaultScreen(display)));
			}
			gc = XCreateGC(display, dst->pxmp, 0 , NULL);
			XSetGraphicsExposures(display, gc, False);

			XCopyArea(display, src->pxmp, dst->pxmp, gc, srcx, srcy, width, height, destx, desty);
			XFreeGC(display, gc);
		}

		if (src->mask != None)
		{
			if (dst->mask == None)
				dst->mask = XCreatePixmap(display, win, dst->w, dst->h, 1);
			mask_gc = XCreateGC(display, dst->mask, 0 , NULL);
			XSetGraphicsExposures(display, mask_gc, False);
			
			XCopyArea(display, src->mask, dst->mask, mask_gc, srcx, srcy, width, height, destx, desty);
			XFreeGC(display, mask_gc);
		}
	
//		dst->h = src->h;
//		dst->w = src->w;
#ifdef __debug__
		XSync(display, False);
		printf("%s() ********* Copied from %p to %p with width = %d, height = %d\n", __func__, src, dst, src->w, src->h);
#endif
		return True;
	}
	return False;
}

Bool image_copy_part_of_image(Window win, px_pixmap* px, const struct Image* img, 
		int srcx, int srcy, int destx, int desty, int width, int height)
{
	Display* display = display_get_Xdisplay();

	if (px != NULL && img != NULL)
	{
		if (img->img != NULL)
		{
			GC gc;
			
			if (px->pxmp == None)
				px->pxmp = XCreatePixmap(display, win, px->w, px->h, 
						DefaultDepth(display, DefaultScreen(display)));

			gc = XCreateGC(display, px->pxmp, 0, NULL);
			XSetGraphicsExposures(display, gc, False);
			
			XPutImage(display, px->pxmp, gc, img->img, srcx, srcy, destx, desty, px->w, px->h);
			XFreeGC(display, gc);
		}

		if (img->mimg != NULL)
		{
			GC gc;
			
			if (px->mask == None)
				px->mask = XCreatePixmap(display, win, px->w, px->h, 1);

			gc = XCreateGC(display, px->mask, 0, NULL);
			XSetGraphicsExposures(display, gc, False);
			
			XPutImage(display, px->mask, gc, img->mimg, srcx, srcy, destx, desty, px->w, px->h);
			XFreeGC(display, gc);
		}
		return True;
	}

	return False;
}

void image_destroy(px_pixmap* pixmap)
{
	Display* display = display_get_Xdisplay();
	if (pixmap->pxmp != None)
	{
		XFreePixmap(display, pixmap->pxmp);
		pixmap->pxmp = None;
	}

	if (pixmap->mask != None)
	{
		XFreePixmap(display, pixmap->mask);
		pixmap->mask = None;
	}
}
